<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use \Exception as Exception;

class CadCargo extends CrudWithParameterBag {
    
    const dataSource              = 'orm00016_cargovendas';
    const dataSourceOperador      = 'orm00016_cargovendas_operador';
    const dataSourceFilial        = 'orm00016_cargovendas_operador_unidade';
    protected $parameterBagColumns = array('NRORG');
    protected $parameterBagColumnsOperador = array('NRORG', 'CDCARGO');
    protected $parameterBagColumnsFilial = array('NRORG', 'CDCARGO', 'CDOPERADOR');
    protected $crudService;
    
              
    public function __construct (ParameterBag $parameterBag, Manager $dataSourceManager, $crudService) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->crudService = $crudService;
    }
    
    public function findCargo(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSource;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request, $response);     
    }
   
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSource;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteCargo(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSource;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findCargoOperador(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceOperador;
        $this->parameterBagColumns = $this->parameterBagColumnsOperador;
        parent::find($request, $response);     
    }
   
    public function saveOperador(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceOperador;
        $dataSet = $request->getDataSet();
        $persistedRows = 0;
        $rows = $dataSet->getRows();
        $rowsToSave = $dataSet->getRows();
        try {
            foreach($rows[0]['CDOPERADOR'] as $operador){
                $rowsToSave[0]['CDOPERADOR'] = $operador;
                $newRequest = new DTO\Request\DataSet (
    	            new DataSet($this->dataSourceName, $rowsToSave) , 
    	            $request->getMethod(),
    	            $request->getRoutePath(),
    	            $request->getUserId()
    	        );
    	        $dataSet = $newRequest->getDataSet();
                $dataSet->setDataSourceName('orm00016_cargovendas_operador');
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            }
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteOperador(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceOperador;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findCargoOperadorUnidade(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceFilial;
        $this->parameterBagColumns = $this->parameterBagColumnsFilial;
        parent::find($request, $response);     
    }
   
    public function saveOperadorUnidade(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceFilial;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteOperadorUnidade(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceFilial;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
