<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;

class RelMapaResumoECF {
    
  protected $entityManager;
  protected $params = array();
  protected $erros = array(
    "operaPerm"=>"Operação não permitida. A emissão do relatório não se enquadra nos parâmetros definidos para a data de processamento.",
    "vendInv"=>"Somatório dos valores totais das Reduções Z da venda líquida não coincide com o valor digitado.",
    "ContInv"=>"Somatório dos contadores dos números das Reduções Z não coincide com o valor digitado.",
    "semDados"=>"Não há dados a serem listados."
  );
  
  const MSG = 'MSG';
  
	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function checkAdqMapaResumo ($conditions){
	    $this->construirParams($conditions);
	    
	    return $this->checkNumRows();
	}
	
	private function construirParams($conditions){
        $params = array();
        foreach ($conditions as $key=>$value) {
            $params[$key] = $value;
        }
        $this->params = $params;
    }
    
    private function checkNumRows (){
        if($this->checkParaVend()){
           return $this->checkParaVend();
        }
        $mapaResumo = $this->executeQuery(self::QRY_MAPA_RESUMO);
        $msg[self::MSG] = $this->erros['semDados'];
        return count($mapaResumo) > 0 ? $mapaResumo : $msg;
    }
    
    private function executeQuery ($query){
        return $this->entityManager->getConnection()->executeQuery($query,$this->params)->fetchAll();
    }
    
    private function checkParaVend(){
        $paraVenda  = $this->executeQuery(self::ADQ_PARVEND);
        if(isset($this->verificarParametrizacao($paraVenda)[self::MSG])){
            return  $this->verificarParametrizacao($paraVenda);
        }
        
        if(isset($paraVenda['IDCONSREDZMECF']) && $this->seAtivo($paraVenda['IDCONSREDZMECF'])){
          return  $this->verificarConsistencia($paraVenda);
        }
    }
    
    private function verificarConsistencia($paraVenda){
      
      $msg = array();
      $media = isset($this->params['MEDIA_VR_LIQ_REDZ']) ? $this->params['MEDIA_VR_LIQ_REDZ'] : '';
      $contador = isset($this->params['MEDIA_NR_REDZ']) ? $this->params['MEDIA_NR_REDZ'] : '';
      $getReducaoZ = $this->executeQuery(self::GET_REDUCAO_Z);
      $getReducaoZ = $this->refatorarArray($getReducaoZ[0]);
      if(empty($media) || $this->valorDiferente($media,$getReducaoZ['VRVENLIQREDZ'])){
          $msg[self::MSG] = $this->erros['vendInv'];
      }else if(empty($contador) || $this->valorDiferente($contador,$getReducaoZ['NRREDREDZ'])){
          $msg[self::MSG] = $this->erros['ContInv'];
      }
      return $msg;
    }
    
    private function valorDiferente($valor1, $valor2){
        return $this->retirarPontoVirgula($valor1) != $this->retirarPontoVirgula($valor2);
    }
    
    private function retirarPontoVirgula ($string){
      return str_replace(array('.',','),"",$string);
    }
    
    private function verificarParametrizacao($params){
      $params = $this->refatorarArray($params[0]);
      $msg = array();
      if(!empty($params['DTPROCVND'])){
          $datas = array('P_DTVENDAINI','P_DTVENDAFIN');
          foreach ($datas as $data) {
            if($this->dataMaior($this->params[$data],$params['DTPROCVND'])){
              if($this->seAtivo($params['IDPERLANCPOSDP'])){
                  if(($this->dataMaior($this->params[$data],$this->somarDiasAData($params['DTPROCVND'],$params['NRDIAPOSLANC'])))){
                     $msg[self::MSG] = $this->erros['operaPerm'];
                  }
              }else{
                $msg[self::MSG] = $this->erros['operaPerm'];
              }
            }else if($this->dataMenor($this->params[$data],$params['DTPROCVND'])){
              if($this->seAtivo($params['IDPERLANCANTDP'])){
                if(($this->dataMenor($this->params[$data],$this->subtrairDiasAData($params['DTPROCVND'],$params['NRDIAANTLANC'])))){
                     $msg[self::MSG] = $this->erros['operaPerm'];
                  }
              }else{
                $msg[self::MSG] = $this->erros['operaPerm'];
              }
            }
          }
      }
      return $msg;
    }
    
    private function seAtivo ($string){
        return $string == "S";
    }
    
    private function somarDiasAData($data,$dias){
        $data = str_replace('/','-',$data);
        $data = date("d-m-Y", strtotime($data." + ".((int)$dias)." day"));
        $data = str_replace('-','/',$data);
        return $data;
    }
    
    private function subtrairDiasAData($data,$dias){
      	$data = str_replace('/','-',$data);
		    $data = date("d-m-Y", strtotime($data." - ".((int)$dias)." day"));
    		$data = str_replace('-','/',$data);
    		return $data;
    }
    
    private function refatorarArray($array){
      $lista = array();
      foreach ($array as $key => $value) {
        $lista[$key] = $value;
      }
      return $lista;
    }
    
    private function dataMaior($dtIni,$dtFim){
     return $this->cnverteEInverterData($dtIni) > $this->cnverteEInverterData($dtFim) ;
    }
    
    private function dataMenor($dtIni,$dtFim){
     return $this->cnverteEInverterData($dtIni) < $this->cnverteEInverterData($dtFim) ;
    }
    
    private function cnverteEInverterData($data){
      $data = explode(" ",$data);
      $data = explode("/",$data[0]);
      
      return $data[2].$data[1].$data[0];
    }
    
    const GET_REDUCAO_Z = "
  SELECT NVL(SUM(NVL(R.NRREDREDZ,0)),0) AS NRREDREDZ,
               NVL(SUM(NVL(R.VRVENBRUREDZ,0) - NVL(R.VRTOTCANREDZ,0) - NVL(R.VRTOTDESREDZ,0)),0) AS VRVENLIQREDZ
      FROM REDUCAOZ R, FILIAL F, CAIXA C, LOJA L
       WHERE R.CDFILIAL = F.CDFILIAL
         AND R.CDFILIAL = C.CDFILIAL
         AND R.CDCAIXA  = C.CDCAIXA
         AND L.CDFILIAL = C.CDFILIAL
         AND L.CDLOJA   = C.CDLOJA
         AND R.NRORG = :NRORG
         AND R.NRORG = F.NRORG
           AND ((INSTR(:P_CDFILIAL, R.CDFILIAL) <> 0) OR (:P_TCDFILIAL ='T'))
           AND ((INSTR(:P_CDCAIXA,  R.CDFILIAL||R.CDCAIXA ) <> 0) OR (:P_TCDCAIXA  = 'T'))
           AND ((INSTR(:P_CDREGIAO, F.CDREGIAO) <> 0) OR (:P_TCDREGIAO ='T'))
           AND ((INSTR(:P_CDLOJA,   L.CDFILIAL||L.CDLOJA  ) <> 0) OR (:P_TCDLOJA   = 'T'))
         AND (R.DTREDUCAOZ BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN)
    ";
    
    const ADQ_PARVEND = "
      SELECT
       IDORIGVENDA, IDTIPMAPRESU,
       DTPROCVND,IDCONSREDZMECF,IDPERLANCANTDP,IDPERLANCPOSDP,
       NRDIAANTLANC, NRDIAPOSLANC
    FROM
       PARAVEND
    WHERE
       CDFILIAL = :CDFILIAL_CURRENT AND NRORG = :NRORG
    ";
    
	  const QRY_MAPA_RESUMO = "
        	SELECT Z.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ           AS DTVENDA,
              TO_NUMBER(Z.NRECFREDZ) AS NRECFREDZ,
              F.CDREGIAO,
              ''               AS CDTIPORECE,
              ''               AS NMTIPORECE,
              LPAD(' ',4,' ')  AS CDCAIXA,
              LPAD(' ',30,' ') AS NMCAIXA,
              MIN(A.MENOR) MENOR,
              MAX(A.MAIOR) MAIOR,
              SUBSTR(:P_NRSEQUENCIA, 1, 20) AS SEQ,
              SUBSTR(:P_NRECF, 1, 20)       AS NUM,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST AS DSENDEFILI,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              SUM(Z.VRVENBRUREDZ) AS VRGRDTOTFIM,
              Z.VRVENBRUREDZ      AS VRGRDTOTINI,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRECFREDZ AS NRECFTURCAIX,
              Z.NRREINIREDZ
            FROM CAIXA C,
              REDUCAOZ Z,
              FILIAL F,
              LOJA L,
              MUNICIPIO MU,
              INSCRESTAD IE,
              (SELECT RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ,
                MIN(RZ.NRPRIMCFREDZ) MENOR,
                MAX(RZ.NRULTCFREDZ) MAIOR
              FROM REDUCAOZ RZ,
                REGIAO RE,
                FILIAL FI,
                CAIXA CA
              WHERE (RZ.NRORG = :NRORG) AND (RE.NRORG = RZ.NRORG )AND (FI.NRORG = RZ.NRORG) AND (CA.NRORG = FI.NRORG) 
              AND(RE.CDREGIAO                     = FI.CDREGIAO )
              AND (FI.CDFILIAL                       = RZ.CDFILIAL )
              AND (RZ.CDFILIAL                       = CA.CDFILIAL )
              AND (RZ.CDCAIXA                        = CA.CDCAIXA )
              AND ((INSTR(:P_CDFILIAL, RZ.CDFILIAL) <> 0)
              OR (:P_TCDFILIAL                       = 'T'))
              AND ((INSTR(:P_CDCAIXA, RZ.CDFILIAL
                ||RZ.CDCAIXA)                       <> 0)
              OR (:P_TCDCAIXA                        = 'T'))
              AND ((INSTR(:P_CDREGIAO, FI.CDREGIAO) <> 0)
              OR (:P_TCDREGIAO                       = 'T'))
              AND ((INSTR(:P_CDLOJA, CA.CDFILIAL
                ||CA.CDLOJA) <> 0)
              OR (:P_TCDLOJA  = 'T'))
              AND (RZ.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
              GROUP BY RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ
              ) A
            WHERE (Z.NRORG = :NRORG) AND (F.NRORG = Z.NRORG) AND (C.NRORG = Z.NRORG)  AND (L.NRORG = C.NRORG) AND (IE.NRORG = F.NRORG)
            AND (:P_AGRUPA                      = 'N' )
            AND (Z.CDFILIAL                       = C.CDFILIAL)
            AND (Z.CDCAIXA                        = C.CDCAIXA)
            AND (F.CDFILIAL                       = Z.CDFILIAL)
            AND (C.CDFILIAL                       = Z.CDFILIAL)
            AND (C.CDCAIXA                        = Z.CDCAIXA)
            AND (Z.CDFILIAL                       = A.CDFILIAL (+))
            AND (Z.NRECFREDZ                      = A.NRECFREDZ (+))
            AND (Z.DTREDUCAOZ                     = A.DTREDUCAOZ (+))
            AND (L.CDFILIAL                       = C.CDFILIAL )
            AND (L.CDLOJA                         = C.CDLOJA )
            AND (F.CDEMPRESA                      = IE.CDEMPRESA )
            AND (F.CDINSCESTA                     = IE.CDINSCESTA)
            AND (IE.CDPAIS                        = MU.CDPAIS )
            AND (IE.SGESTADO                      = MU.SGESTADO )
            AND (IE.CDMUNICIPIO                   = MU.CDMUNICIPIO)
            AND ((INSTR(:P_CDFILIAL, Z.CDFILIAL) <> 0)
            OR (:P_TCDFILIAL                      = 'T'))
            AND ((INSTR(:P_CDCAIXA, Z.CDFILIAL
              ||Z.CDCAIXA)                       <> 0)
            OR (:P_TCDCAIXA                       = 'T'))
            AND ((INSTR(:P_CDREGIAO, F.CDREGIAO) <> 0)
            OR (:P_TCDREGIAO                      = 'T'))
            AND ((INSTR(:P_CDLOJA, C.CDFILIAL
              ||C.CDLOJA) <> 0)
            OR (:P_TCDLOJA = 'T'))
            AND (Z.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
            GROUP BY Z.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ,
              Z.NRECFREDZ,
              F.CDREGIAO,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              Z.VRVENBRUREDZ,
              Z.VRVENBRUREDZ,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRREINIREDZ
            UNION
            SELECT M.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ           AS DTVENDA,
              TO_NUMBER(Z.NRECFREDZ) AS NRECFREDZ,
              F.CDREGIAO,
              M.CDTIPORECE,
              TR.NMTIPORECE,
              LPAD(' ',4,' ')  AS CDCAIXA,
              LPAD(' ',30,' ') AS NMCAIXA,
              MIN(A.MENOR) MENOR,
              MAX(A.MAIOR) MAIOR,
              SUBSTR('', 1, 20) AS SEQ,
              SUBSTR('', 1, 20) AS NUM,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST AS DSENDEFILI,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              SUM(Z.VRVENBRUREDZ) AS VRGRDTOTFIM,
              Z.VRVENBRUREDZ      AS VRGRDTOTINI,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRECFREDZ AS NRECFTURCAIX,
              Z.NRREINIREDZ
            FROM MOVCAIXA M,
              TIPORECE TR,
              CAIXA C,
              REDUCAOZ Z,
              FILIAL F,
              LOJA L,
              MUNICIPIO MU,
              INSCRESTAD IE,
              (SELECT RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ,
                MIN(RZ.NRPRIMCFREDZ) MENOR,
                MAX(RZ.NRULTCFREDZ) MAIOR
              FROM REDUCAOZ RZ,
                REGIAO RE,
                FILIAL FI,
                CAIXA CA
              WHERE  (RZ.NRORG = :NRORG) AND (RE.NRORG = RZ.NRORG )AND (FI.NRORG = RZ.NRORG) AND (CA.NRORG = FI.NRORG) 
              AND (RE.CDREGIAO                     = FI.CDREGIAO )
              AND (FI.CDFILIAL                       = RZ.CDFILIAL )
              AND (RZ.CDFILIAL                       = CA.CDFILIAL )
              AND (RZ.CDCAIXA                        = CA.CDCAIXA )
              AND ((INSTR(:P_CDFILIAL, RZ.CDFILIAL) <> 0)
              OR (:P_TCDFILIAL                       = 'T'))
              AND ((INSTR(:P_CDCAIXA, RZ.CDFILIAL
                ||RZ.CDCAIXA)                       <> 0)
              OR (:P_TCDCAIXA                        = 'T'))
              AND ((INSTR(:P_CDREGIAO, FI.CDREGIAO) <> 0)
              OR (:P_TCDREGIAO                       = 'T'))
              AND ((INSTR(:P_CDLOJA, CA.CDFILIAL
                ||CA.CDLOJA) <> 0)
              OR (:P_TCDLOJA  = 'T'))
              AND (RZ.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
              GROUP BY RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ
              ) A
            WHERE (Z.NRORG = :NRORG) AND (F.NRORG = Z.NRORG) AND (C.NRORG = Z.NRORG)  AND (L.NRORG = C.NRORG) AND (IE.NRORG = F.NRORG) 
            AND (:P_AGRUPA                      = 'S' )
            AND (M.CDFILIAL                       = Z.CDFILIAL )
            AND (M.CDCAIXA                        = Z.CDCAIXA )
            AND (M.DTMOVIMCAIXA                   = Z.DTREDUCAOZ)
            AND (C.CDFILIAL                       = Z.CDFILIAL)
            AND (C.CDCAIXA                        = Z.CDCAIXA)
            AND (F.CDFILIAL                       = Z.CDFILIAL)
            AND (Z.CDFILIAL                       = A.CDFILIAL (+))
            AND (Z.NRECFREDZ                      = A.NRECFREDZ (+))
            AND (Z.DTREDUCAOZ                     = A.DTREDUCAOZ(+))
            AND (L.CDFILIAL                       = C.CDFILIAL )
            AND (L.CDLOJA                         = C.CDLOJA )
            AND (TR.CDTIPORECE                    = M.CDTIPORECE)
            AND (F.CDEMPRESA                      = IE.CDEMPRESA )
            AND (F.CDINSCESTA                     = IE.CDINSCESTA)
            AND (IE.CDPAIS                        = MU.CDPAIS )
            AND (IE.SGESTADO                      = MU.SGESTADO )
            AND (IE.CDMUNICIPIO                   = MU.CDMUNICIPIO)
            AND (M.IDTIPOMOVIVE                  IN ('E', 'T') )
            AND (M.VRMOVIVEND                    <> 0 )
            AND ((INSTR(:P_CDFILIAL, Z.CDFILIAL) <> 0)
            OR (:P_TCDFILIAL                      = 'T'))
            AND ((INSTR(:P_CDCAIXA, Z.CDFILIAL
              ||Z.CDCAIXA)                       <> 0)
            OR (:P_TCDCAIXA                       = 'T'))
            AND ((INSTR(:P_CDREGIAO, F.CDREGIAO) <> 0)
            OR (:P_TCDREGIAO                      = 'T'))
            AND ((INSTR(:P_CDLOJA, C.CDFILIAL
              ||C.CDLOJA) <> 0)
            OR (:P_TCDLOJA = 'T'))
            AND (Z.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
            GROUP BY M.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ,
              Z.NRECFREDZ,
              F.CDREGIAO,
              M.CDTIPORECE,
              TR.NMTIPORECE,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              Z.VRVENBRUREDZ,
              Z.VRVENBRUREDZ,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRREINIREDZ
            UNION
            SELECT M.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ           AS DTVENDA,
              TO_NUMBER(Z.NRECFREDZ) AS NRECFREDZ,
              F.CDREGIAO,
              'CVE',
              'CV Emitido',
              LPAD(' ',4,' ')  AS CDCAIXA,
              LPAD(' ',30,' ') AS NMCAIXA,
              MIN(A.MENOR) MENOR,
              MAX(A.MAIOR) MAIOR,
              SUBSTR(:P_NRSEQUENCIA, 1, 20) AS SEQ,
              SUBSTR(:P_NRECF, 1, 20)       AS NUM,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST AS DSENDEFILI,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              SUM(Z.VRVENBRUREDZ) AS VRGRDTOTFIM,
              Z.VRVENBRUREDZ      AS VRGRDTOTINI,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRECFREDZ AS NRECFTURCAIX,
              Z.NRREINIREDZ
            FROM MOVCAIXA M,
              TIPORECE TR,
              CAIXA C,
              FILIAL F,
              LOJA L,
              REDUCAOZ Z,
              MUNICIPIO MU,
              INSCRESTAD IE,
              (SELECT RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ,
                MIN(RZ.NRPRIMCFREDZ) MENOR,
                MAX(RZ.NRULTCFREDZ) MAIOR
              FROM REDUCAOZ RZ,
                REGIAO RE,
                FILIAL FI,
                CAIXA CA
              WHERE (RZ.NRORG = :NRORG) AND (RE.NRORG = RZ.NRORG )AND (FI.NRORG = RZ.NRORG) AND (CA.NRORG = FI.NRORG)
              AND (RE.CDREGIAO                     = FI.CDREGIAO )
              AND (FI.CDFILIAL                       = RZ.CDFILIAL )
              AND (RZ.CDFILIAL                       = CA.CDFILIAL )
              AND (RZ.CDCAIXA                        = CA.CDCAIXA )
              AND ((INSTR(:P_CDFILIAL, RZ.CDFILIAL) <> 0)
              OR (:P_TCDFILIAL                       = 'T'))
              AND ((INSTR(:P_CDCAIXA, RZ.CDFILIAL
                ||RZ.CDCAIXA)                       <> 0)
              OR (:P_TCDCAIXA                        = 'T'))
              AND ((INSTR(:P_CDREGIAO, FI.CDREGIAO) <> 0)
              OR (:P_TCDREGIAO                       = 'T'))
              AND ((INSTR(:P_CDLOJA, CA.CDFILIAL
                ||CA.CDLOJA) <> 0)
              OR (:P_TCDLOJA  = 'T'))
              AND (RZ.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
              GROUP BY RZ.CDFILIAL,
                RZ.NRECFREDZ,
                RZ.DTREDUCAOZ
              ) A
            WHERE (Z.NRORG = :NRORG) AND (F.NRORG = Z.NRORG) AND (C.NRORG = Z.NRORG)  AND (L.NRORG = C.NRORG) AND (IE.NRORG = F.NRORG)
            AND (:P_AGRUPA                      = 'S' )
            AND (M.CDFILIAL                       = Z.CDFILIAL )
            AND (M.CDCAIXA                        = Z.CDCAIXA )
            AND (M.DTMOVIMCAIXA                   = Z.DTREDUCAOZ)
            AND (C.CDFILIAL                       = Z.CDFILIAL)
            AND (C.CDCAIXA                        = Z.CDCAIXA)
            AND (F.CDFILIAL                       = Z.CDFILIAL)
            AND (L.CDFILIAL                       = C.CDFILIAL )
            AND (L.CDLOJA                         = C.CDLOJA )
            AND (F.CDEMPRESA                      = IE.CDEMPRESA )
            AND (F.CDINSCESTA                     = IE.CDINSCESTA)
            AND (IE.CDPAIS                        = MU.CDPAIS )
            AND (IE.SGESTADO                      = MU.SGESTADO )
            AND (IE.CDMUNICIPIO                   = MU.CDMUNICIPIO)
            AND (TR.CDTIPORECE                    = M.CDTIPORECE)
            AND (M.IDTIPOMOVIVE                   = 'S' )
            AND (TR.IDTIPORECE                    = '7' )
            AND (M.VRMOVIVEND                    <> 0 )
            AND (M.IDTIPOMOVIVE                  IN ('E', 'T'))
            AND ((INSTR(:P_CDFILIAL, M.CDFILIAL) <> 0)
            OR (:P_TCDFILIAL                      = 'T'))
            AND ((INSTR(:P_CDCAIXA, M.CDFILIAL
              ||M.CDCAIXA)                       <> 0)
            OR (:P_TCDCAIXA                       = 'T'))
            AND ((INSTR(:P_CDREGIAO, F.CDREGIAO) <> 0)
            OR (:P_TCDREGIAO                      = 'T'))
            AND ((INSTR(:P_CDLOJA, L.CDFILIAL
              ||L.CDLOJA) <> 0)
            OR (:P_TCDLOJA = 'T'))
            AND (Z.DTREDUCAOZ BETWEEN TO_DATE(SUBSTR(:P_DTVENDAINI, 1, 10), 'DD/MM/YYYY') AND TO_DATE(SUBSTR(:P_DTVENDAFIN, 1, 10), 'DD/MM/YYYY'))
            GROUP BY M.CDFILIAL,
              F.CDEMPRESA,
              F.NMFILIAL,
              Z.DTREDUCAOZ,
              Z.NRECFREDZ,
              F.CDREGIAO,
              M.CDTIPORECE,
              TR.NMTIPORECE,
              F.NMRAZSOCFILI,
              F.CDINSCESTA,
              MU.SGESTADO,
              MU.NMMUNICIPIO,
              IE.DSENDEIEST,
              F.NRINSJURFILI,
              Z.VRTOTCANREDZ,
              Z.VRTOTDESREDZ,
              Z.VRGTREDZ,
              Z.VRVENBRUREDZ,
              Z.VRVENBRUREDZ,
              Z.NRPRIMCFREDZ,
              Z.NRULTCFREDZ,
              Z.NRREDREDZ,
              Z.VRISSREDZ,
              Z.NRSERIEREDZ,
              Z.NRREINIREDZ
            ORDER BY 1,2,3,4,5,6,7,8
	";
}